# .addons/eventscripts/quizer/quizer.py

# *********************************************
# ** DO NOT EDIT THIS FILE - EDIT CONFIG.CFG **
# *********************************************
# **** ADD YOUR QUESTIONS TO QUESTIONS.TXT ****
# *********************************************

import es
import random
import gamethread
import cfglib
import cPickle
import popuplib
import cmdlib
from path import path

''' Addon Information '''
info = es.AddonInfo()
info.name = 'Quizer'
info.version = '1.0'
info.basename = 'quizer'
info.author = 'Craziest'
info.description = 'Players will be asked a question each round'

''' Config Generator '''
cfg = cfglib.AddonCFG(es.getAddonPath("quizer") + "/config/config.cfg")

cfg.text("******************************")
cfg.text("****** QUIZER SETTINGS *******")
cfg.text("********* craziest ***********")

enableRewards = cfg.cvar('quizer_rewards', 1, 'Should players get rewards for answering correctly')
restartQuestions = cfg.cvar('quizer_question_restart', 0, 'Should questions be changed every round or left until they are answered? (1 for each round)')
enablePopup = cfg.cvar('quizer_player_popup', 1, 'Allow players to see their stats through a popup')
playerTries = cfg.cvar('quizer_player_tries', 0, 'How many tries does the player have (0 = Unlimited)')
resetTries = cfg.cvar('quizer_reset_tries', 1, 'Should the player\'s tries reset each round start (0 = No, 1 = Yes). If no, they will restart when the question ends')

cfg.text(' ')
cfg.text("******************************")
cfg.text("****** COMMAND HANDLING ******")
cfg.text("******************************")

answerCommand = cfg.cvar('quizer_answer_command', '!answer', 'The command that should be used for answering questions')
questionCommand = cfg.cvar('quizer_question_command', '!question', 'The command that should be used by player to tell them the question')
popupCommand = cfg.cvar('quizer_popup_command', '!quizme', 'If popup is enabled above, what should the command to see your stats be')

cfg.text(' ')
cfg.text("******************************")
cfg.text("****** DATABASE HANDLING *****")
cfg.text("******************************")

savePlayers = cfg.cvar('quizer_save_players', 1, 'Save players achievments if the script loads/unloads or the server crashes')
savePlayers_frequency = cfg.cvar('quizer_save_players_frequency', 'round', 'How often should players be saved (either map or round)')

cfg.text(' ')
cfg.text("******************************")
cfg.text("****** REWARD HANDLING *******")
cfg.text("******************************")

reward_level1 = cfg.cvar('quizer_reward_one', 'weapon_smokegrenade', 'What should the player get at 5 questions correctly answered')
reward_level2 = cfg.cvar('quizer_reward_two', 'weapon_flashbang', 'What should the player get at 10 questions correctly answered')
reward_level3 = cfg.cvar('quizer_reward_three', 'weapon_hegrenade', 'What should the player get at 15 questions correctly answered')
reward_level4 = cfg.cvar('quizer_reward_four', 'weapon_deagle', 'What should the player get at 20 questions correctly answered')

cfg.write()

''' Variable Assigning Filed '''
questionFile = es.getAddonPath("quizer") + '/config/questions.txt'
qDict = {} # as in question dict
aDict = {} # as in answer dict
commands = [popupCommand, questionCommand, answerCommand]
prefix = '#lightgreen[#defaultQuizer#lightgreen]#green'

''' Load / Unload '''
def load():
    global questionRunning
    global aDict
    
    # Execute the config file
    cfg.execute()
    # Message the plugin is loaded
    msg('Loaded')
    # Get the questions and answers
    qFile = open(questionFile, 'r')
    lines = qFile.readlines()
    qFile.close()
    for line in filter(lambda line: not line.startswith('//'), lines):
            line = line.strip('"')
            if line:
                fq = line.find('"')
                if fq:
                    q = line[:fq]
                    a = line[fq:].replace('"', '').strip()
                    qDict[q] = a
                else:
                    es.dbgmsg(0, "Incorrect Syntax! 'Question''Answer' (No spaces between the middle '')")
            else:
                es.dbgmsg(0, "Incorrect Syntax! 'Question''Answer' (No spaces between the middle '')")
    
    # Register the command
    cmdlib.registerSayCommand(answerCommand, answerCmd, "Quizer Plugin Commands")
    cmdlib.registerSayCommand(questionCommand, questionCmd, "Quizer Plugin Commands")
    if enablePopup:
        cmdlib.registerSayCommand(popupCommand, popupCmd, "Quizer Plugin Commands")
    # No questions are running
    questionRunning = 0
    # Open the database
    dbOpen()
    # Set all the players (that are currently on the server) correct answer count to none
    for x in es.getUseridList():
        # Get the steamid...
        steamid = es.getplayersteamid(x)
        # Add their steamid if it doesn't exist
        if not aDict.has_key(steamid):
            aDict[steamid] = {}
            aDict[steamid]['total'] = 0
            aDict[steamid]['spree'] = 0
            aDict[steamid]['correct'] = 0
        # Set their tries to 0
        aDict[steamid]['tries'] = 0
    
def unload():
    # Message it's unloaded
    msg('Unloaded')
    # Un-register the command
    for x in commands:
        cmdlib.unregisterSayCommand(x)
    # Save the database
    dbSave()
    
''' Wrapper Functions '''
def msg(text):
    es.msg('#multi', prefix + ' ' + text)
    
def tell(userid, text):
    es.tell(userid, '#multi', prefix + ' ' + text)

''' Event Functions '''
def round_start(ev):
    global questionRunning
    global question
    global aDict
    
    # Get all the players
    for x in es.getUseridList():
        # Get their steamids
        steamid = es.getplayersteamid(x)    
        if resetTries or restartQuestions:
            # Reset the player's tries
            aDict[steamid]['tries'] = 0
    
    # If a question isn't running
    if not questionRunning or restartQuestions:
        # Choose a question
        question = random.choice(qDict.keys())
        # Tell users a vote is running
        es.msg('#multi', prefix + ' Question #default~ #lightgreen' + question)
        # A question is waiting to be answered...
        questionRunning = 1
        
    # Or if it is then...
    else:
        # The same question is still in progress
        msg('Still not answered! #default~ #lightgreen%s' % question)
        
def round_end(ev):
    global question
    # Cancel questions if the user has selected to restart questions every round
    if restartQuestions:
        question = None
    
    # Save the players if frequency is every round
    if savePlayers:
        if savePlayers_frequency == 'round':
            dbSave()
    
    
def player_activate(ev):
    global aDict
    # Get the players userid
    userid = int(ev['userid'])
    # Plus his steamid
    steamid = es.getplayersteamid(userid)
    # Add him to the dictionary if he doesn't exist
    if not aDict.has_key(steamid):
        aDict[steamid] = {}
        aDict[steamid]['total'] = 0
        aDict[steamid]['spree'] = 0
        aDict[steamid]['tries'] = 0
        aDict[steamid]['correct'] = 0
        
    # Tell the player the commmands
    tell(userid, 'is running on this server. Type #lightgreen!quizme#green to see your stats and more information.')
    
def map_end(ev):
    # Save the players if frequency is every map
    if savePlayers:
        if savePlayers_frequency == 'map':
            dbSave()
  
''' Command Functions '''
def questionCmd(userid, args):
    # If a question is running...
    if questionRunning:
        # ... tell the player which question is running
        tell(userid, 'Current Question #default~ #lightgreen%s' % question)
    else:
        # ... tell him there are no question running at this moment
        tell(userid, 'No questions are currently running')
    
def answerCmd(userid, args):
    global aDict
    
    # Get the player's userid
    userid = es.getcmduserid()
    # His name...
    name = es.getplayername(userid)
    # His steamid
    steamid = es.getplayersteamid(userid)
    
    # If a question is running
    if questionRunning:
        if args:
            # Get the players answer
            answer = str(args)
            # Get the correct answer
            correctAnswer = qDict[question]
            # If he hasn't used all of his tries or a player has unlimited tries
            if aDict[steamid]['tries'] < playerTries or playerTries == 0:
                # Add 1 to his total points
                aDict[steamid]['total'] += 1
                # If his answer is correct
                if answer.lower() == correctAnswer.lower():
                    # Tell the player he answered correctly
                    tell(userid, 'Congratulations! #default~ #lightgreen%s is the correct answer!' % correctAnswer)
                    # Add a correct answer to the player's correct answer count
                    aDict[steamid]['correct'] += 1
                    # Cancel the question
                    cancelQuestion()
                    # Broadcast to all players the player has answered correctly
                    gamethread.delayed(2, msg, '#lightgreen%s#green answered correctly #default~ #greenHe has answered #lightgreen%s#green questions out of #lightgreen%s#green correctly.' % (name, aDict[steamid]['correct'], aDict[steamid]['total']))

                    # Add 1 to the player's correct answer spree, his rewards will be based on this
                    aDict[steamid]['spree'] += 1
                    # Check if the use should get any rewards
                    checkRewards(userid, aDict[steamid]['spree'])
                    # Delete all tries he has
                    aDict[steamid]['tries'] = 0
                
                # Or if the answer isn't correct
                else:
                    # Add 1 to their tries
                    aDict[steamid]['tries'] += 1
                    # Tell him it's not correct
                    if playerTries > 0:
                        # Get how many tries he has left
                        triesLeft = playerTries - aDict[steamid]['tries']
                        # If he has 1 more try warn him
                        if triesLeft == '0':
                            tell(userid, 'This is your last try #default~ #lightgreenBe careful!')
                        # Else tell him how many has he got left
                        else:
                            tell(userid, 'You have not answered correctly (#lightgreen%s#green tries left) #default~#lightgreen Try again!' % triesLeft)
                    
                    elif playerTries == 0:
                        tell(userid, 'You have not answered correctly #default~#lightgreen Better luck next time!')
                        
                    # Cancel his winning spree (if he has any)
                    aDict[steamid]['spree'] = 0
            else:
                # Tell him he used all of his tries
                tell(userid, 'You have used all of your tries this round.')
                # Cancel his winning spree (if he has any)
                aDict[steamid]['spree'] = 0
        else:
            # Tell him the correct syntax
            tell(userid, 'The correct syntax is: #lightgreen%s <answer>#default ~#green Type #lightgreen%s #greenif you do not know the current question.' % (answerCommand, questionCommand))
    else:
        # Tell him a question isn't running
        tell(userid, 'No questions are currently running.')
        
def popupCmd(userid, args):
    if not args:
        # Get steamid
        steamid = es.getplayersteamid(userid)
        # Create the popup
        rankPopup = popuplib.create('rankspopup')
        # Add lines
        rankPopup.addline('** Crazy\'s Quizme Menu **')
        rankPopup.addline(' ')
        rankPopup.addline('->1 - Your statistics')
        rankPopup.addline(' ')
        rankPopup.addline(' Total Answered: %s' % aDict[steamid]['total'])
        rankPopup.addline(' Correct Answered: %s' % aDict[steamid]['correct'])
        rankPopup.addline(' Current Spree: %s' % aDict[steamid]['spree'])
        rankPopup.addline(' ')
        rankPopup.addline('->2 - Help')
        rankPopup.addline(' ')
        rankPopup.addline(' Answer the questions with:')
        rankPopup.addline(' >> %s <answer>' % answerCommand)
        rankPopup.addline(' To view the current question:')
        rankPopup.addline(' >> %s <question>' % questionCommand)
        rankPopup.addline(' ')
        rankPopup.addline('->8. Close')
        # Send the popup
        popuplib.send('rankspopup', userid)
    
        # Define which function handles choices
        rankPopup.menuselect = rankPopup_menuselect
    # If the player wrote in another player
    elif es.getuserid(args):
        # Get steamid
        steamid = es.getplayersteamid(es.getuserid(args))
        # Create the popup
        otherpopup = popuplib.create('otherpopup')
        # Calculate the players correct answered procent
        percent = 0
        # Add lines
        otherpopup.addline('** Crazy\'s Quizme Menu **')
        otherpopup.addline(' ')
        otherpopup.addline('->1 - %s\'s statistics' % es.getplayername(es.getuserid(args)))
        otherpopup.addline(' ')
        otherpopup.addline(' Total Answered: %s' % aDict[steamid]['total'])
        otherpopup.addline(' Correct Answered: %s' % aDict[steamid]['correct'])
        otherpopup.addline(' Current Spree: %s' % aDict[steamid]['spree'])
        otherpopup.addline(' ')
        otherpopup.addline('->2 - Help')
        otherpopup.addline(' ')
        otherpopup.addline(' Answer the questions with:')
        otherpopup.addline(' >> %s <answer>' % answerCommand)
        otherpopup.addline(' To view the current question:')
        otherpopup.addline(' >> %s <question>' % questionCommand)
        otherpopup.addline(' ')
        otherpopup.addline('->8. Close')
        # Send the popup
        popuplib.send('otherpopup', userid)
    
        # Define which function handles choices
        rankPopup.menuselect = rankPopup_menuselect
        
    else:
        # tell him he didn't put in the correct syntax
        tell(userid, 'Invalid Format! #default~ #greenEither #lightgreen!quizme <player> #greenor #lightgreenno arguments#green.')
    
''' Popup Functions '''
def rankPopup_menuselect(userid, choice, popupid):
    if choice:
        # Delete the popup, so it can be updated
        popuplib.delete('rankspopup')
        popuplib.delete('otherpopup')
            
''' Custom Functions '''
def checkRewards(userid, spree):
    if enableRewards:
        # If a player has a winning spree of 5 correct answers
        if spree == 5:
            gamethread.delayed(5, tell, (userid, 'You are on a winning spree! You are rewarded for your knowledge.'))
            gamethread.delayed(5, es.server.queuecmd, 'es_xgive %s %s' % (userid, reward_level1))
        # If a player has a winning spree of 10 correct answers
        elif spree == 10:
            gamethread.delayed(5, tell, (userid, 'You are on a higher winning spree! You are rewarded for your knowledge.'))
            gamethread.delayed(5, es.server.queuecmd, 'es_xgive %s %s' % (userid, reward_level2))
        # If a player has a winning spree of 15 correct answers
        elif spree == 15:
            gamethread.delayed(5, tell, (userid, 'You are on a higher winning spree! You are richly rewarded for your knowledge.'))
            gamethread.delayed(5, es.server.queuecmd, 'es_xgive %s %s' % (userid, reward_level3))
        # If a player has a winning spree of 20 correct answers
        elif spree == 20:
            gamethread.delayed(5, tell, (userid, 'The highest winning spree! You are handsomely rewarded for your knowledge.'))
            gamethread.delayed(5, es.server.queuecmd, 'es_xgive %s %s' % (userid, reward_level4))
    
            
def cancelQuestion():
    global questionRunning
    global aDict
    
    # Cancel any questions that are running
    questionRunning = 0
    
    # Get all the players
    for x in es.getUseridList():
        # Plus their steamids
        steamid = es.getplayersteamid(x)
        # Set their tries to 0
        aDict[steamid]['tries'] = 0
        
 
''' Database Handling '''   
def dbSave():
    # Save the player database
    p = open(es.getAddonPath("quizer") + '/ranks.db', 'wb')
    cPickle.dump(aDict, p)
    p.close()
    
def dbOpen():
    # Open the player database
    global aDict
    p = es.getAddonPath("quizer") + '/ranks.db'
    if path(p).isfile():
        f = open(p, 'rb')
        aDict = cPickle.load(f)
        f.close()    